import hashlib

def xor_bytes(a, b):
    # Apply XOR byte by byte; zip stops at the shortest block (here 16 bytes)
    return bytes(x ^ y for x, y in zip(a, b))

def remove_pkcs7_padding(data):
    padding_length = data[-1]
    return data[:-padding_length]

def decrypt(ciphertext, key):
    block_size = 16
    # Calculate the hash of the key and use only the first 16 bytes for XOR
    key_hash = hashlib.sha256(key).digest()
    key_block = key_hash[:block_size]
    
    decrypted = b""
    for i in range(0, len(ciphertext), block_size):
        block = ciphertext[i:i+block_size]
        decrypted += xor_bytes(block, key_block)
    
    # Remove PKCS#7 padding
    return remove_pkcs7_padding(decrypted).decode('utf-8')

def decode_encrypted_file(encrypted, key):
    decrypted_text = decrypt(encrypted, key)
    
    # The original blockchain has a length of 324
    blockchain_length = 324
    # The token was inserted at the middle, at index 162.
    token_length = len(decrypted_text) - blockchain_length
    flag = decrypted_text[162:162 + token_length]
    
    return flag

# Example usage:
if __name__ == "__main__":
    # These are the data printed by the original application
    key = b'Z\xa3\xf6\xd4\xdb\x8a\x9c\x10\x84\xf8\xb6\xb0:\x1c\xce\xca\xbfX\x96\x9d\x87\tm\xd6\xbe4a\xc5\xd5\x91^\x98'
    encrypted_blockchain = b'o\xd3\xefR\xf9@\x01(\xebg\xf0\xed\xc5K\xcc\x87>\x83\xe8\x05\xaeB\x02)\xbe2\xa7\xe2\xc6\x19\x97\xd2h\xd9\xeaW\xfcF\x05+\xb20\xf4\xb7\x97N\x9f\xd5e\xd2\xeb\x02\xf9FVx\xeb`\xa2\xe4\xc7M\x9c\x85p\xd1\xe8P\xffOR.\xee5\xa6\xb6\x94\x18\x9e\x84i\xd6\xbaQ\xf9\x14\x00.\xee3\xa2\xe4\x93\x1c\x9a\x83h\x87\xea\x01\xf8FV"\xbbe\xa2\xb6\x93N\x9b\xd5l\xd4\xbaU\xa8\x17Z.\xbdb\xa6\xe5\xc7H\xcc\x8fk\xcc\xe8P\xa9C\x07(\xbd<\xf4\xe7\xc6\x18\x99\xd0;\x82\xe0\x01\xa8GQz\xeb6\xaf\xe5\xc7H\xca\x829\xd2\xa8\t\xa9\x19 O\xcc\x7f\xf4\xb9\x99\x1e\xc5\xe9n\xb2\x8a\x08\x9c\x1f1y\xde5\xe7\xb6\xae"\xf6\xe37\xac\xe8\x12\xfeO\x00S\xd5u\xd5\xaf\x9b7\xf4\xcc\x1f\xaa\x87T\xff\x15\x07)\xeb1\xa4\xa8\x92D\x9b\xd5k\x87\xebQ\xac\x17U,\xb93\xf2\xe4\x92I\x96\xd3l\x87\xbcQ\xa8DV*\xb3f\xf4\xb6\xdbM\x9e\x82e\xd6\xbcY\xfdNQ"\xbc4\xf3\xe4\xc4I\x97\x858\xd8\xbcQ\xabDZ+\xe83\xa7\xb7\xc2I\x9f\xd7d\x80\xee\x01\xafG\x07,\xbf7\xa3\xe6\x95\x1c\x99\xd7m\xd0\xb9Y\xacCU}\xece\xaf\xb3\x90P\x9e\x86n\x82\xbaQ\xae\x14W/\xec`\xf0\xe7\xc0N\x9a\x81m\xd6\xe8Y\xa8BU(\xbd6\xa1\xe1\xc1J\x97\x81j\x87\xbbV\xfeBW~\xbf3\xa6\xb6\xc2\x18\x97\xd0>\xd4\xeaU\xf8\x13\x01-\xeb2\xf2\xe5\xc5\x19\xac\xb4'
    
    # Decode and extract the injected flag (token)
    flag = decode_encrypted_file(encrypted_blockchain, key)
    print("\nFLAG:", flag)
